using System.Net.Http.Headers;
using System.Net.Http.Json;
using FitConnect.Services.Models;

namespace FitConnect.Services;

public class OAuthService {
    private readonly string _tokenUrl;

    public OAuthService(string tokenUrl) {
        _tokenUrl = tokenUrl;
    }

    /// <summary>
    ///     Requesting an OAuth token from the FitConnect API.
    ///     <para>
    ///         You can get the Client ID and Client Secret from the FitConnect Self Service portal
    ///         under <br />
    ///         https://portal.auth-testing.fit-connect.fitko.dev
    ///     </para>
    /// </summary>
    /// <param name="clientId">Your client Id</param>
    /// <param name="clientSecret">Your client Secret</param>
    /// <param name="scope">Scope if needed</param>
    /// <returns>The received token or null</returns>
    public async Task<OAuthAccessToken?> AuthenticateAsync(string clientId, string clientSecret,
        string? scope = null) {
        var client = new HttpClient();
        client.DefaultRequestHeaders.Accept.Add(
            MediaTypeWithQualityHeaderValue.Parse("application/json"));

        var requestContent = new Dictionary<string, string> {
            { "grant_type", "client_credentials" },
            { "client_id", clientId },
            { "client_secret", clientSecret }
        };

        if (scope != null)
            requestContent["scope"] = scope;

        var content = new FormUrlEncodedContent(requestContent);

        var request = new HttpRequestMessage(HttpMethod.Post, _tokenUrl) {
            Content = content,
            Method = HttpMethod.Post
        };

        var response = await client.SendAsync(request);

        return await response.Content.ReadFromJsonAsync<OAuthAccessToken>();
    }
}
